import requests
from typing import Callable, TypeVar, get_type_hints, Type
import xwire.transport
from xwire.common import decorator_utils
from xwire.rest._private.utils import get_cookies
from xwire.rest._private.proxy.decorators import resource


T = TypeVar('T')


def _do_get(url: str, return_class: Type[T], **kwargs) -> T:
    response = requests.get(url, timeout=5, **kwargs)
    if response.status_code >= 400:
        response.raise_for_status()
    elif return_class is not None:
        return xwire.transport.json.deserialize(response.json(), return_class)
    else:
        return None


def get(path: str):
    def decorator(func: Callable[..., T]) -> Callable[..., T]:
        return_type = get_type_hints(func)['return']

        def wrapper(self, *args):
            resource_metadata = decorator_utils.get_decorator_metadata(self, resource)
            url = self.base_url + resource_metadata['resource_path'] + path
            return _do_get(url, return_type, cookies=get_cookies(self))

        return wrapper
    return decorator



from typing import TypeVar, Type
from xwire.common import decorator_utils

T = TypeVar('T')


def resource(resource_path: str):
    def decorator(cls: Type[T]) -> Type[T]:
        decorator_utils.set_decorator_metadata(cls, resource, {
            'resource_path': resource_path,
            'cookies': None
        })
        return cls
    return decorator




import inspect
from typing import TypeVar, Dict, Callable
from xwire.common import decorator_utils
from xwire.rest._private.proxy.decorators import cookies, resource

T = TypeVar('T')


def get_cookies(resource_obj: T) -> Dict[str, str]:
    resource_metadata = decorator_utils.get_decorator_metadata(resource_obj, resource)
    cookies_method: Callable[..., Dict[str, str]] = resource_metadata['cookies']
    if cookies_method is None:
        methods = inspect.getmembers(resource_obj, inspect.ismethod)
        for _, method in methods:
            if decorator_utils.has_decorator(method, cookies):
                cookies_method = method
                resource_metadata['cookies'] = cookies_method
                decorator_utils.set_decorator_metadata(resource_obj, resource, resource_metadata)
                return cookies_method()
        return {}
    else:
        return cookies_method()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xwire',
 'xwire.rest',
 'xwire.rest._private',
 'xwire.rest._private.proxy',
 'xwire.rest._private.proxy.decorators',
 'xwire.rest.public']

package_data = \
{'': ['*']}

install_requires = \
['pytest-mock>=3.5.1,<4.0.0',
 'requests>=2.25.1,<3.0.0',
 'xwire.common>=0.1.1,<0.2.0',
 'xwire.transport>=0.1.3,<0.2.0']

setup_kwargs = {
    'name': 'xwire.rest',
    'version': '0.1.5',
    'description': '',
    'long_description': None,
    'author': 'tylerlaberge',
    'author_email': 'tylerlaberge@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
